﻿using System.Windows;
using System.IO;
using System.Collections.Generic;
using System.Windows.Input;

/*
 *  Created by Shane D. Anderson, 2015.
 *  Last updated May 25, 2015.
 */

namespace RemoveBOM
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // BOMs are: UTF8; UTF16 little-endian; UTF16 big-endian; UTF32 little-endian; UTF32 bid-endian.
        string[] BOMs = { "EFBBBF", "FFFE", "FEFF", "FFFE0000", "0000FEFF" };

        public MainWindow() {
            InitializeComponent();
        }

        private void buttonRemoveBOM_Click(object sender, RoutedEventArgs e) {
            int hexIn;
            string hex;
            bool NoBOMsFound = true;

            // Proceeds in two steps: 1) Read; 2) Overwrite;
            foreach (string str in listBoxFiles.Items) {
                if (File.Exists(str) == true) {
                    FileStream fs = new FileStream(str, FileMode.Open, FileAccess.Read);
                    List<byte> byteStreamOut = new List<byte>();
                    string BOMString = "";

                    for (int i = 0; (hexIn = fs.ReadByte()) != -1; i++) {
                        byteStreamOut.Add((byte)hexIn);

                        // Handle removing any possible BOMs. We're done after reading and checking 4 bytes.
                        if (i < 4)
                        {
                            hex = string.Format("{0:X2}", hexIn);
                            BOMString += hex;

                            for (int j = 0; j < BOMs.Length; j++)
                            {
                                if (BOMString.CompareTo(BOMs[j]) == 0)
                                {
                                    byteStreamOut.Clear();
                                    NoBOMsFound = false;
                                }
                            }
                        }
                        else
                            break;
                    }
                    fs.Close();

                    // Write over the file
                    fs = new FileStream(str, FileMode.Create, FileAccess.Write);
                    fs.Write(byteStreamOut.ToArray(), 0, byteStreamOut.Count);
                    fs.Close();
                }
            }

            if (listBoxFiles.Items.Count == 0) {
                MessageBox.Show(Properties.Resources.ResourceManager.GetString("NoFilesSelected"));
            } else if (NoBOMsFound) {
                MessageBox.Show(Properties.Resources.ResourceManager.GetString("NoBOMsFound"));
            } else {
                MessageBox.Show(Properties.Resources.ResourceManager.GetString("BOMsRemoved"));
            }
        }

        private void listBoxFiles_DragEnter(object sender, DragEventArgs e) {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effects = DragDropEffects.All;
            else
                e.Effects = DragDropEffects.None;
        }

        private void listBoxFiles_Drop(object sender, DragEventArgs e) {
            string[] s = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            int i;
            FileAttributes attr;
            bool previouslyAdded = false;

            for (i = 0; i < s.Length; i++) {
                previouslyAdded = false;
                attr = File.GetAttributes(s[i]);
                if (!((attr & FileAttributes.Directory) == FileAttributes.Directory)) {

                    foreach (string str in listBoxFiles.Items) {
                        if (str.CompareTo(s[i]) == 0)
                            previouslyAdded = true;
                    }

                    if (previouslyAdded == false)
                        listBoxFiles.Items.Add(s[i]);
                }
            }
        }

        private void listBoxFiles_KeyDown(object sender, System.Windows.Input.KeyEventArgs e) {
            if (e.Key == Key.Delete) {
                while (listBoxFiles.SelectedItem != null) {
                    listBoxFiles.Items.Remove(listBoxFiles.SelectedItem);
                }
            }
        }
    }
}